glm_cidr

Package Version Hex Docs

This is a simple library to assist with answering these basic questions:

I’m sure there are other interesting questions related to cidr.

Feel free to fork and/or submit PR’s for additional functionality.

gleam add glm_cidr

See tests for how to use.

Further documentation can be found at https://hexdocs.pm/glm_cidr.

Development

gleam check # Check the project
gleam test  # Run the tests
Search Document